<?php

namespace Drupal\orejime\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\Yaml\Yaml;

/**
 * Class OrejimeSettingsForm.
 *
 * @ingroup orejime
 */
class OrejimeSettingsForm extends ConfigFormBase {

  /**
   * Returns a unique string identifying the form.
   *
   * @return string
   *   The unique string identifying the form.
   */
  public function getFormId() {
    return 'orejime_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['orejime.settings'];
  }


  /**
   * Defines the settings form for Orejime entities.
   *
   * @param array $form
   *   An associative array containing the structure of the form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   *
   * @return array
   *   Form definition array.
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $form['orejime_settings'] = [
      '#type' => 'vertical_tabs',
      '#title' => t('Settings'),
    ];
    $form['account'] = [
      '#type' => 'details',
      '#group' => 'orejime_settings',
      '#title' => $this->t('General settings'),
    ];
    $form['language'] = [
      '#type' => 'details',
      '#group' => 'orejime_settings',
      '#title' => $this->t('Language settings'),
    ];
    $form['account']['cookie_name'] = [
      '#title' => t('Cookie Name'),
      '#type' => 'textfield',
      '#default_value' => $this->getSettings('cookie_name', 'orejime'),
      '#required' => TRUE,
      '#description' => t('You can customize the name of the cookie that Orejime uses for storing user consent decisions.'),
    ];
    $form['account']['expires_after_days'] = [
      '#title' => t('Cookie expires after days'),
      '#type' => 'textfield',
      '#default_value' => $this->getSettings('expires_after_days', '365'),
      '#required' => TRUE,
      '#description' => t('You can set a custom expiration time for the Orejime cookie, in days.'),
    ];
    $form['account']['privacy_policy'] = [
      '#title' => t('Privacy Policy'),
      '#type' => 'textfield',
      '#default_value' => $this->getSettings('privacy_policy', 'orejime'),
      '#required' => TRUE,
      '#description' => t(' You must provide a link to your privacy policy page'),
    ];
    $form['account']['must_consent'] = [
      '#title' => t('Must consent'),
      '#type' => 'checkbox',
      '#default_value' => $this->getSettings('must_consent', FALSE),
      '#required' => FALSE,
      '#description' => t('"mustConsent" is set to true, Orejime will directly display the consent manager modal and not allow the user to close it before having actively consented or declined the use of third-party apps.'),
    ];
    $form['account']['must_notice'] = [
      '#title' => t('Must notice'),
      '#type' => 'checkbox',
      '#default_value' => $this->getSettings('must_notice', FALSE),
      '#required' => FALSE,
      '#description' => t('If "mustNotice" is set to true, Orejime will display the consent notice and not allow the user to close it before having actively consented or declined the use of third-party apps. Has no effect if mustConsent is set to true.'),
    ];
    $form['account']['implicit_consent'] = [
      '#title' => t('Implicit consent'),
      '#type' => 'checkbox',
      '#default_value' => $this->getSettings('implicit_consent', FALSE),
      '#required' => FALSE,
      '#description' => t('If "implicitConsent" is set to true, Orejime will automatically accept cookies if the user continues his navigation on the website after the first page. If you enable this, you must warn the user of this behavior in the notice window. You can do that easily by overriding translation strings (see below).'),
    ];
    $form['account']['analytics'] = [
      '#title' => t('Codes UA à gérer'),
      '#type' => 'textfield',
      '#default_value' => $this->getSettings('analytics'),
      '#required' => FALSE,
      '#description' => t('Liste here UA used in the website.'),
    ];
    $form['account']['orejime_css'] = [
      '#title' => t('CSS Orejime'),
      '#type' => 'textfield',
      '#default_value' => $this->getSettings('orejime_css'),
      '#required' => FALSE,
      '#description' => t('Set here the url if you want a different CSS or use external link'),
    ];
    $form['account']['orejime_js'] = [
      '#title' => t('JS Orejime'),
      '#type' => 'textfield',
      '#default_value' => $this->getSettings('orejime_js'),
      '#required' => FALSE,
      '#description' => t('Set here the url if you want a different js for orejime or use external link'),
    ];
    $form['account']['debug'] = [
      '#title' => t('Debug Mode'),
      '#type' => 'checkbox',
      '#default_value' => $this->getSettings('debug'),
      '#required' => FALSE,
      '#description' => t('Set Orejime in debug mode to have a few stuff logged in the console, like warning about missing translations.'),
    ];
    $form['language']['texts'] = [
      '#title' => t('Texts of Orejime'),
      '#type' => 'textarea',
      '#rows' => 35,
      '#default_value' => $this->getSettings('texts'),
      '#required' => FALSE,
      '#description' => t('Translation file. Example here : https://raw.githubusercontent.com/empreinte-digitale/orejime/master/src/translations/en.yml'),
    ];
    if (\Drupal::service('module_handler')->moduleExists('content_translation')) {
      $form_state->set(['content_translation', 'key'], 'language');
      $form['language'] += content_translation_enable_widget('orejime_service', 'orejime_service', $form, $form_state);
    }
    return parent::buildForm($form, $form_state);
  }

  /**
   * Implements _form_validate().
   * --------------
   * @param $form
   * @param $form_state
   */
  function validateForm(array &$form, FormStateInterface $form_state) {
    try {
      Yaml::parse($form_state->getValue('texts'));
    } catch (\Exception $e) {
      $form_state->setErrorByName('texts', $e->getMessage());
    }
    parent::validateForm($form, $form_state);
  }

  /**
   * Allow to get settings or default value
   * --------------
   * @param $name
   * @param string $default
   * @return string
   */
  function getSettings($name, $default = '') {
    $site_config = $this->config('orejime.settings');
    return $site_config->get($name) ?: $default;
  }

  /**
   * Form submission handler.
   *
   * @param array $form
   *   An associative array containing the structure of the form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $this->config('orejime.settings')
      ->set('cookie_name', $form_state->getValue('cookie_name'))
      ->set('expires_after_days', $form_state->getValue('expires_after_days'))
      ->set('privacy_policy', $form_state->getValue('privacy_policy'))
      ->set('must_consent', $form_state->getValue('must_consent'))
      ->set('must_notice', $form_state->getValue('must_notice'))
      ->set('implicit_consent', $form_state->getValue('implicit_consent'))
      ->set('analytics', $form_state->getValue('analytics'))
      ->set('orejime_css', $form_state->getValue('orejime_css'))
      ->set('orejime_js', $form_state->getValue('orejime_js'))
      ->set('texts', $form_state->getValue('texts'))
      ->set('debug', $form_state->getValue('debug'))
      ->save();
    parent::submitForm($form, $form_state);
  }


}
